/*
 * Copyright (c) 2004-2006 Weswit s.r.l., Via Campanini, 6 - 20124 Milano, Italy.
 * All rights reserved.
 * www.lightstreamer.com
 *
 * This software is the confidential and proprietary information of
 * Weswit s.r.l.
 * You shall not disclose such Confidential Information and shall use it
 * only in accordance with the terms of the license agreement you entered
 * into with Weswit s.r.l.
 */

using System.Collections;
using Lightstreamer.Interfaces.Data;

namespace Nms.Server.Test {

	/// <summary>
	/// This Data Adapter accepts a limited set of item names (the names starting
	/// with "item") and listens to a (simulated) stock quotes feed, waiting for
	/// update events. The events pertaining to the currently subscribed items
	/// are then forwarded to Lightstreamer.
	/// This example demonstrates how a Data Adapter could interoperate with
	/// a broadcast feed, which sends data for all items. Many other types of feeds
	/// may exist, with very different behaviours.
	/// </summary>
	public class NmsServerAdapter : IDataProvider, IExternalFeedListener {

		private IDictionary _subscribedItems;
		private ExternalFeedSimulator _myFeed;

		private IItemEventListener _listener;

		public NmsServerAdapter() {
			_subscribedItems= new Hashtable();
			
			_myFeed= new ExternalFeedSimulator();
		}
		
		// ////////////////////////////////////////////////////////////////////////
		// IDataProvider methos

		public void Init(IDictionary parameters, string configFile) {
			_myFeed.SetFeedListener(this);
			_myFeed.Start();
		}

		public void SetListener(IItemEventListener eventListener) {
			_listener= eventListener;
		}

		public void Subscribe(string itemName) {
			if (!itemName.StartsWith("item")) 
				throw new SubscriptionException("Unexpected item: " + itemName);

			lock (_subscribedItems) {
				if (_subscribedItems.Contains(itemName)) return;

				_subscribedItems[itemName]= false;
			}

			_myFeed.SendCurrentValues(itemName);
		}
		
		public void Unsubscribe(string itemName) {
			if (!itemName.StartsWith("item")) 
				throw new SubscriptionException("Unexpected item: " + itemName);

			lock (_subscribedItems) {
				_subscribedItems.Remove(itemName);
			}
		}

		public bool IsSnapshotAvailable(string itemName) {
			if (!itemName.StartsWith("item")) 
				throw new SubscriptionException("Unexpected item: " + itemName);

			return true;
		}
	
		// ////////////////////////////////////////////////////////////////////////
		// IExternalFeedListener methods

		public void OnEvent(string itemName, 
			IDictionary currentValues,
			bool isSnapshot) {
			lock (_subscribedItems) {
				if (!_subscribedItems.Contains(itemName)) return;

				bool started = (bool) _subscribedItems[itemName];
				if (!started) {
					if (!isSnapshot) 
						return;
					
					_subscribedItems[itemName]= true;
				} 
				else {
					if (isSnapshot) {
						isSnapshot = false;
					}
				}
			}

			if (_listener != null)
				_listener.Update(itemName, currentValues, isSnapshot);
		}
	}
	
}